
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.all"

//*****************************************************************************************//
//                                      Object                                             //
//*****************************************************************************************//

void Object::export (::ofstream &output) {
	//YOU DO THIS...
}

void Object::log () {
	//The type (it's implicit... it will be different for subclasses)...
	::log ("\nGeneral Object");

	//The faces...
	loopVector (faceIndex, faces)
		Face &face = *((*faces) [faceIndex]);
		::log ("\n\tFace %d:", faceIndex);
		::log ("\n\t\tTexture \"%s\":", face.textureName);
		GamePointCollection &points = *(face.points);
		loopVector (pointIndex, &points)
			GamePoint &point = *(points [pointIndex]);
			::log ("\n\t\t%d: point [%3.2f,%3.2f,%3.2f] normal: [%3.2f,%3.2f,%3.2f] texture: [%3.2f,%3.2f].",
				pointIndex, point.x, point.y, point.z, point.nx, point.ny, point.nz, point.tx, point.ty);
			//Access point's x, y, z, nx, ny, nz, tx, ty
		endloop
	endloop
}

//*****************************************************************************************//
//                                       World                                             //
//*****************************************************************************************//

void World::setup () {
	//Nothing to do so far.
}

void World::wrapup () {
}

World::World () {
	objects = new ObjectCollection;
	startPosition.x = startPosition.y = startPosition.z = 0; //In case there is no "info_player_start"...
};

World::~World () {
	deleteObjectCollection (objects);
};

char realInputFilename [MAX_PATH];

World *World::read () {
	//Prompts for a ".map" file, reads it in, creates a World object, invokes
	//"add (properties,faces) for each Object, and returns the World object (or NULL if
	//not successful).

	//Read in the worldcraft ".map" data.
	char *worldcraftFilename = promptForMapRead ();
	if (worldcraftFilename == NULL) return NULL; //User must have cancelled.
	strcpy (realInputFilename, worldcraftFilename);
	WorldcraftGroupCollection *groups = worldcraftReader (worldcraftFilename);

	//Create a World object and loop through the Worldcraft groups adding each object to the world.
	World *world = new World ();
	loopVector (groupIndex, groups)
		WorldcraftGroup *group = (*groups) [groupIndex];
		char *type = (char *) get (group, "classname");
		if (type == NULL) {::log ("\nCould not extract the class name, ignored item..."); continue;}
		WorldcraftObjectCollection *objects = (WorldcraftObjectCollection *) get (group, "objects");
		loopVector (objectIndex, objects)
			WorldcraftObject &object = *((*objects) [objectIndex]);
			object.type = type; object.properties = group;
			world->add (object);
		endloop
	endloop

	//Now that we're done, delete the worldcraft data and return the world (it's made copies of everything).
	deleteWorldcraftGroupCollection (groups);
	return world;
}

FaceCollection *World::copyFaces (FaceCollection *faces) {
	//Make a new face collection.
	FaceCollection *newFaces = new FaceCollection;
	loopVector (faceIndex, faces)
		Face &face = *((*faces) [faceIndex]);
		//Make a new point collection.
		GamePointCollection *newPoints = new GamePointCollection;
		//Make new points.
		GamePointCollection &points = *(face.points);
		loopVector (pointIndex, &points)
			GamePoint &point = *(points [pointIndex]);
			GamePoint *newPoint = new GamePoint;
			*newPoint = point; //Copy the data.
			newPoints->push_back (newPoint);
		endloop
		//Make a new texture name.
		char *newTextureName = new char [strlen (face.textureName) + 1];
		strcpy (newTextureName, face.textureName);
		//Make a new face with the above point collection and texture (and the plane) and add to the collection.
		Face *newFace = new Face (face.plane, newPoints, newTextureName);
		newFaces->push_back (newFace);
	endloop
	return newFaces;
}

void World::add (WorldcraftObject &worldcraftObject) {
	//Create a NON-worldcraft object corresponding to the worldcraft object.

	//For debugging, let's see what the worldcraft object looks like.
	::log ("\n\nInput object..."); worldcraftObject.log ();

	if (strcmp (worldcraftObject.type, "worldspawn") == 0) {
		//It's a standard static world object.
		Object *object = new Object (copyFaces (worldcraftObject.faces));
		objects->push_back (object);
		//And then let's see what the normal object looks like...
		::log ("\n\nOutput object..."); object->log ();
	} else if (strcmp (worldcraftObject.type, "info_player_start") == 0) {
		//It's special, record the position in the world object.
		char *origin = (char *) get (worldcraftObject.properties, "origin");
		sscanf (origin, "%f %f %f", &startPosition.x, &startPosition.y, &startPosition.z);
	} else if (strcmp (worldcraftObject.type, "Environment") == 0) {
		//It's probably special too...
		//... for you to do...
	} else if (strcmp (worldcraftObject.type, "Vehicle") == 0) {
		//It's a special world object (how about making it a subclass of Object).
		//... for you to do...
	} else {
		::log ("\nUnknown object type \"%s\" ignored.", worldcraftObject.type);
	}

	//And then let's see what the normal object looks like...
}

void World::export (char *filename) {
	//Write out your world (perhaps asking different components to write themselves out).

	//FOR YOU TO DO....... THOUGH HERE'S A START...
	::ofstream output (filename);
	if (output.bad ()) {
		MessageBox (NULL, NULL, "Unable to create file", MB_OK);
	}
	
	//The simple stuff.
	output << "\nWorld version 1";
	output << startPosition.x << " " << startPosition.y << " " << startPosition.z;

	//The objects: A version WITH loop macros.
	loopVector (objectIndex, objects)
		Object &object = *((*objects) [objectIndex]);
		object.export (output);
	endloop
}

void World::log () {
	//For debugging, it might be useful to output the entire world into the log file.

	//The simple stuff first.
	::log ("\n\nWorld...\nStart position [%3.2f,%3.2f,%3.2f]", startPosition.x, startPosition.y, startPosition.z);

	//The objects: A version WITH loop macros.
	loopVector (objectIndex, objects)
		Object &object = *((*objects) [objectIndex]);
		object.log ();
	endloop
	
	/*
	//The objects: A version WITHOUT loop macros.
	for (long objectIndex = 0; objectIndex < objects->size (); objectIndex++) {
		Object &object = *((*objects) [objectIndex]);
		object.log ();
	}
	*/
}
