
//95.402 Wilf LaLonde: Student development Worldcraft to Student world converter.
//This program creates world (or level) files for use by the game engine.

//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include "includes.all"

//*****************************************************************************************//
//                                        Main                                             //
//*****************************************************************************************//

enum MenuItem {DifficultyEasy, DifficultyChallenging, DifficultyImpossible,
	DisplayModeWireframe, DisplayModeTextured, RunWorld, Quit};

MenuItem difficulty; MenuItem displayMode;
bool wireframe = false, fullscreen = false;
char filename [256];

void resizeWindow (int width, int height) {
	//Setup a new viewport.
	glViewport (0, 0, width, height);

	//Setup a new perspective matrix.
	GLdouble verticalFieldOfViewInDegrees = 40;
	GLdouble aspectRatio = height == 0 ? 1.0 : (GLdouble) width / (GLdouble) height;
	GLdouble nearDistance = 1.0;
	GLdouble farDistance = 2000.0;

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective (verticalFieldOfViewInDegrees, aspectRatio, nearDistance, farDistance);

	//Get back to default mode.
	glMatrixMode (GL_MODELVIEW);
}

void displayWindow () {
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	game->draw ();
	glutSwapBuffers ();
}

void idle () {
	game->tick ();
	glutPostRedisplay ();
}

void visibilityChanged (int visible) {
	glutIdleFunc (visible == GLUT_VISIBLE ? idle : NULL);
}

inline bool keyIsDown (long key) {return (GetAsyncKeyState (key) & 0x8000) != 0;}
#define VK_ALT VK_MENU

void specialKeyPressed (int character, int x, int y) {
	switch (character) {
		case GLUT_KEY_F1:
			glutPostRedisplay ();
			break;
		case GLUT_KEY_F2:
			glutPostRedisplay ();
			break;
		case GLUT_KEY_UP: {
			//A partial example...
			if (keyIsDown (VK_CONTROL)) {
			} else if (keyIsDown (VK_ALT)) {
			} else {
			}
			glutPostRedisplay ();
			break;}
		case GLUT_KEY_DOWN: {
			glutPostRedisplay ();
			break;}
		case GLUT_KEY_RIGHT:
			glutPostRedisplay ();
			break;
		case GLUT_KEY_LEFT:
			glutPostRedisplay ();
			break;
		case GLUT_KEY_PAGE_UP:
			glutPostRedisplay ();
			break;
		case GLUT_KEY_PAGE_DOWN:
			glutPostRedisplay ();
			break;
	}
}

#define escapeCharacter 27

void normalKeyPressed (unsigned char character, int x, int y) {
	//Handle the key and then force a redisplay.
	switch (character) {
		case escapeCharacter:
			exit (0);
		case ' ':
			//Open door perhaps?
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9': {
			//Run some tests perhaps?
			long code = character - '0';
			break;}
		case 'w':
		case 'W':
			wireframe = !wireframe;
			glPolygonMode (GL_FRONT_AND_BACK, wireframe ? GL_LINE : GL_FILL);
			break;
		default:
			log ("\nHit normal key %d (%x).", character, character);
	}

	glutPostRedisplay ();
}

void mousePressed (int button, int state, int x, int y) {
	if (button == GLUT_LEFT_BUTTON) {
		if (state == GLUT_DOWN) {
			//Start something at mouse coordinates x and y.
		} else {//state == GLUT_UP
			//End something at mouse coordinates x and y.
		}
	} else if (button == GLUT_MIDDLE_BUTTON) {
		if (state == GLUT_DOWN) {
			//Start something at mouse coordinates x and y.
		} else {//state == GLUT_UP
			//End something at mouse coordinates x and y.
		}
	} else {//button == GLUT_RIGHT_BUTTON
		if (state == GLUT_DOWN) {
			//Start something at mouse coordinates x and y.
		} else {//state == GLUT_UP
			//End something at mouse coordinates x and y.
		}
	}
	log ("\nPressed %s mouse button at %d@%d.", button == GLUT_LEFT_BUTTON ? "LEFT" : 
		(button == GLUT_MIDDLE_BUTTON ? "MIDDLE" : "RIGHT"), x, y);
}

void mouseMoved (int x, int y) {
	log ("\nMoved mouse to %d@%d.", x, y);
}

#define matches(a,b) strlen (b) >= strlen (a) && memcmp (a,b,strlen (a)) == 0

void parseParameters (int parametersSize, char **parameters) {
	bool optionFound = false;
	for (long i = 1; i < parametersSize; i++) {//Skip program name.
		log ("\n%d: Consider \"%s\".", i, parameters [i]);
		if (matches ("-w", parameters [i])) {
			log ("\nMatched -w");
			wireframe = true; optionFound = true;
		} else if (matches ("-f", parameters [i])) {
			log ("\nMatched \"-f\", extracting from \"%s\".", parameters [i]);
			log ("\nAssigning into filename [0]");
			filename [0] = 20;
			log (" WORKED...");
			strcpy (filename, parameters [i]+2);
			log (" COPY WORKED TOO...");
			log ("\nRead filename \"%s\".", filename);
			optionFound = true;
		} else 
			log ("\nFailed to match.");
	}
	if (!optionFound) {
		printf ("\nusage: builder [-options] -fFilename");
		printf ("\n  -w         :: wireframe");
		printf ("\n  -f         :: file name; e.g., -fc:\\test\\data");
		exit (1);
	}
}

void genericMenuHandler (int item) {
	switch (item) {
		case RunWorld: 
			if (game->currentWorld != NULL) {
				game->currentWorld->unload;
				delete game->currentWorld;
			}
			game->currentWorld = World::load ();
			break;
		case Quit:
			exit (0);
			break;
		default:
			log ("\nUnknown generic menu selection %d (%x).", item, item);
	}
}

void difficultyMenuHandler (int item) {
	switch (item) {
		case DifficultyEasy:
		case DifficultyChallenging:
		case DifficultyImpossible:
			difficulty = (MenuItem) item; 
			break;
		default:
			log ("\nUnknown difficulty %d (%x).", item, item);
	}
}

void displayModeMenuHandler (int item) {
	switch (item) {
		case DisplayModeWireframe:
			wireframe = true;
			glPolygonMode (GL_FRONT_AND_BACK, wireframe ? GL_LINE : GL_FILL);
			break;
		case DisplayModeTextured:
			wireframe = false;
			glPolygonMode (GL_FRONT_AND_BACK, wireframe ? GL_LINE : GL_FILL);
			break;
		default:
			log ("\nUnknown display mode %d (%x).", item, item);
	}
    glutPostRedisplay ();
}

void createMenus () {
  int difficultyMenu = glutCreateMenu (difficultyMenuHandler);
  glutAddMenuEntry ("Easy", DifficultyEasy);
  glutAddMenuEntry ("Hard", DifficultyChallenging);
  glutAddMenuEntry ("Impossible", DifficultyImpossible);

  int displayModeMenu = glutCreateMenu (displayModeMenuHandler);
  glutAddMenuEntry ("Wireframe", DisplayModeWireframe);
  glutAddMenuEntry ("Textured", DisplayModeTextured);

  int genericMenu = glutCreateMenu (genericMenuHandler);
  glutAddSubMenu ("Difficulty...", difficultyMenu);
  glutAddSubMenu ("Mode...", displayModeMenu);
  glutAddMenuEntry ("Run World", RunWorld);
  glutAddMenuEntry ("Quit", Quit);

  glutAttachMenu (GLUT_RIGHT_BUTTON);
}

int main (int parametersSize, char **parameters) {
	//Process the command line (if any)...
	log ("\nStarting...");
	//parseParameters (parametersSize, parameters);

	//Setup general facilities.
	glutInitDisplayMode (GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH | GLUT_DOUBLE | GLUT_STENCIL | GLUT_MULTISAMPLE);
	glutInitWindowSize (800, 600);
	glutInit (&parametersSize, parameters);
	if (fullscreen) /*glutEnterGameMode ()*/; else {glutCreateWindow ("game engine"); createMenus ();}

	//Specify function handlers.
	glutDisplayFunc (displayWindow); glutReshapeFunc (resizeWindow); glutKeyboardFunc (normalKeyPressed);
	glutSpecialFunc (specialKeyPressed); glutMouseFunc (mousePressed); glutMotionFunc (mouseMoved);
	glutVisibilityFunc (visibilityChanged);

	//Setup our own facilities.
	Game::setup ();

	//Rumble
	glutMainLoop ();

	//Cleanup
	Game::wrapup ();
	return 0;
}
