
//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>    
#include <windows.h>
#include <commdlg.h>
#include <direct.h>
#include "fileDialog.h"

//*****************************************************************************************//
//                                     File Dialogs                                        //
//*****************************************************************************************//


//*****************************************************************************************//
//                                       Private                                           //
//*****************************************************************************************//

char lastMapDirectory [_MAX_DIR] = {'.','.','\\','M','A','P','S','\0'};
char lastWorldDirectory [_MAX_DIR] = {'.','.','\\','W','O','R','L','D','S','\0'};
char lastTextureDirectory [_MAX_DIR] = {'.','.','\\','T','E','X','T','U','R','E','S','\0'};
char lastDirectory [_MAX_DIR] = {'.','.','\\','M','A','P','S','\0'};

char *directories [4] = {lastMapDirectory, lastWorldDirectory, lastTextureDirectory, lastDirectory};
char lastFilename [MAX_PATH]; 

void setupInternalFilter (char *internalFilter, char *userFilter) {
	//Filter format: pairs of name, filter strings with an extra string delimiter in the end.
	long filterSize = strlen (userFilter) + 1;
	internalFilter [0] = '\0'; 
	memcpy (&internalFilter [1], userFilter, filterSize + 1);
	internalFilter [filterSize + 2] = '\0';
}

void setupQueryStructure (OPENFILENAME &queryStructure, char *filter) {
	char filenameBuffer [MAX_PATH]; strcpy (filenameBuffer, filter);
	char fileFilter [1024]; setupInternalFilter (&fileFilter [0], filter);
	if (lastDirectory [0] == '\0') {_getcwd (lastDirectory, MAX_PATH);}

	queryStructure.lStructSize = sizeof (OPENFILENAME);     
	queryStructure.hwndOwner = NULL;     
	queryStructure.hInstance = GetModuleHandle (NULL);         
	queryStructure.lpstrFilter = fileFilter;     
	queryStructure.lpstrCustomFilter = NULL;     
	queryStructure.nMaxCustFilter = 0;    
	queryStructure.nFilterIndex = 0;     
	queryStructure.lpstrFile = filenameBuffer;     
	queryStructure.nMaxFile = MAX_PATH;    
	queryStructure.lpstrFileTitle = NULL;     
	queryStructure.nMaxFileTitle = MAX_PATH;     
	queryStructure.lpstrInitialDir = lastDirectory;    
	queryStructure.lpstrTitle = NULL;     
	queryStructure.Flags = 0;     
	queryStructure.nFileOffset = 0;     
	queryStructure.nFileExtension = 0;     
	queryStructure.lpstrDefExt = NULL;     
	queryStructure.lCustData = 0;     
	queryStructure.lpfnHook = NULL;     
	queryStructure.lpTemplateName = NULL;     
}

char *promptForReadFile (DirectoryChoice choice, char *filter) {
	useDirectory (choice);
	OPENFILENAME query; setupQueryStructure (query, filter);      
	if (GetOpenFileName (&query)) {         
		strcpy (lastFilename, query.lpstrFile);   
		memorizeDirectoryUsed (choice); 
		return &lastFilename [0];
	}     
	return NULL;
}

char *promptForWriteFile (DirectoryChoice choice, char *filter) {
	useDirectory (choice);
	OPENFILENAME query; setupQueryStructure (query, filter);      
	if (GetSaveFileName (&query)) {       
		strcpy (lastFilename, query.lpstrFile);  
		memorizeDirectoryUsed (choice); 
		return &lastFilename [0];
	}     
	return NULL;
}

//*****************************************************************************************//
//                                       Public                                            //
//*****************************************************************************************//

void useDirectory (DirectoryChoice choice) {
	strcpy (lastDirectory, directories [choice]);
}
void memorizeDirectoryUsed (DirectoryChoice choice) {
	strcpy (directories [choice], lastDirectory);
}

char *promptForMapRead () {return promptForReadFile (MapChoice, "*.map");}
char *promptForMapWrite () {return promptForWriteFile (MapChoice, "*.map");}

char *promptForWorldRead () {return promptForReadFile (WorldChoice, "*.wrl");}
char *promptForWorldWrite () {return promptForWriteFile (WorldChoice, "*.wrl");}

char *promptForTextureRead () {return promptForReadFile (TextureChoice, "*.*");}
char *promptForTextureWrite () {return promptForWriteFile (TextureChoice, "*.*");}

char *promptForAnyRead () {return promptForReadFile (AnyChoice, "*.*");}
char *promptForAnyWrite () {return promptForWriteFile (AnyChoice, "*.*");}