
//*****************************************************************************************//
//                                  Worldcraft Reader                                      //
//*****************************************************************************************//

//*****************************************************************************************//
//                                      Includes                                           //
//*****************************************************************************************//

#ifndef worldcraftReaderModule
#define worldcraftReaderModule 

//*****************************************************************************************//
//                                    Worldcraft Reader                                    //
//*****************************************************************************************//

//The reader reads in raw worldcraft data that describes objects (brushes they call them) in 
//terms of planes (that we'll call a worldcraft side plane since it is not at all what a 
//standard plane looks like). The reader reads all this information in, converts the side 
//plane data into coordinates for faces and returns a vector of worldcraft objects (a map 
//that contains  everything worldcraft provided as key/value pairs (both keys and values are
//strings) except for one additional keys not provided by worldcraft:
//
//		"objects" with an ObjectCollection as value (this in turn contains a FaceCollection).
//
//It is the user's responsibility to delete all the keys and values when a worldcraft
//object is deleted. Use deleteWorldcraftGroup (aWorldCraftObject) to do this properly 
//for you.

//*****************************************************************************************//
//                                    WorldcraftGroup                                     //
//*****************************************************************************************//

//The worldcraft group is a dictionary/map.
typedef map <char *, void *> WorldcraftGroup;
//typedef WorldcraftGroup::iterator WorldcraftGroupIterator;
typedef map <char *, void *>::iterator WorldcraftGroupIterator;

inline void deleteWorldcraftGroup (WorldcraftGroup *collection) {
	for (WorldcraftGroupIterator next = collection->begin (); next != collection->end (); ++next) {
		char *key = next->first; void *value = next->second;
		delete [] key; //It's a string.
		if (strcmp (key, "objects") == 0)
			delete (ObjectCollection *) value; //It's a collection of objects.
		else 
			delete [] (char *) value; //It's another string.
	} 
	delete collection; 
}

//*****************************************************************************************//
//                               WorldcraftGroupCollection                               //
//*****************************************************************************************//

//The worldcraft group collection is a vector of worldcraft groups (each a dictionary/map).

declareCollection (WorldcraftGroup);
//typedef vector <WorldcraftGroup *> WorldcraftCollection;
//typedef vector <WorldcraftGroup>::iterator WorldcraftCollectionIterator;
//inline void deleteWorldcraftCollection (WorldcraftCollection *collection) {
//}

//*****************************************************************************************//
//                                       The API                                           //
//*****************************************************************************************//

WorldcraftGroupCollection *worldcraftReader (char *filename);

#endif